/**
 * Copyright (c) 2017-present, Facebook, Inc.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree. An additional grant
 * of patent rights can be found in the PATENTS file in the same directory.
 *
 * 
 */

'use strict';
Object.defineProperty(exports, '__esModule', {value: true});

var _rsocketTcpClient = require('rsocket-tcp-client');

var _events = require('events');
var _events2 = _interopRequireDefault(_events);
var _net = require('net');
var _net2 = _interopRequireDefault(_net);
var _rsocketFlowable = require('rsocket-flowable');
function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : {default: obj};
}

/**
                                                                                                                                                  * A TCP transport server.
                                                                                                                                                  */
class RSocketTCPServer {
  constructor(options, encoders) {
    this._emitter = new _events2.default();
    this._encoders = encoders;
    this._options = options;
  }

  start() {
    return new _rsocketFlowable.Flowable(subscriber => {
      let server;
      const onClose = () => {
        if (server) {
          server.close();
        }
        subscriber.onComplete();
      };
      const onError = error => subscriber.onError(error);
      const onConnection = socket => {
        subscriber.onNext(
          new _rsocketTcpClient.RSocketTcpConnection(socket, this._encoders)
        );
      };
      subscriber.onSubscribe({
        cancel: () => {
          if (!server) {
            return;
          }
          server.removeListener('connection', onConnection);
          server.removeListener('error', onError);
          this._emitter.removeListener('close', onClose);
          server.close();
          server = null;
        },
        request: n => {
          if (!server) {
            const factory = this._options.serverFactory ||
              _net2.default.createServer;
            server = factory(onConnection);
            server.listen(this._options.port, this._options.host);
            server.on('error', onError);
            this._emitter.on('close', onClose);
          }
        },
      });
    });
  }

  stop() {
    this._emitter.emit('close');
  }
}
exports.default = RSocketTCPServer;
